<?php

declare(strict_types=1);

namespace FiloBlu\Checkproducts\Model\System;

use Magento\Catalog\Model\Product;
use Magento\Eav\Model\Entity\TypeFactory;
use Magento\Eav\Model\ResourceModel\Entity\Attribute\CollectionFactory;
use Magento\Framework\Data\OptionSourceInterface;

/**
 *
 */
class ProductAttributes implements OptionSourceInterface
{
    /**
     * @var \Magento\Eav\Model\ResourceModel\Entity\Attribute\CollectionFactory
     */
    private $attributeCollectionFactory;

    /**
     * @var \Magento\Eav\Model\Entity\TypeFactory
     */
    private $typeFactory;

    /**
     * @param \Magento\Eav\Model\ResourceModel\Entity\Attribute\CollectionFactory $attributeCollectionFactory
     * @param \Magento\Eav\Model\Entity\TypeFactory $typeFactory
     */
    public function __construct(CollectionFactory $attributeCollectionFactory,
        TypeFactory $typeFactory){
        $this->attributeCollectionFactory = $attributeCollectionFactory;
        $this->typeFactory = $typeFactory;
    }

    /**
     * @return array
     */
    public function toOptionArray()
    {
        $options = [];
        $attributes = $this->getAttributesToCheck();
        $attributeGroups = array_unique($attributes->getColumnValues('attribute_group_name'));
        foreach ($attributeGroups as $attributeGroupName) {
            $groupedOptions = [];
            $groupAttributes = $attributes->getItemsByColumnValue('attribute_group_name', $attributeGroupName);
            foreach ($groupAttributes as $attribute) {
                $attributeLabel = $attribute->getFrontendLabel() ? $attribute->getFrontendLabel(
                ) : $attribute->getAttributeCode();
                $groupedOptions[] = [
                    'label' => $attributeLabel,
                    'value' => $attribute->getAttributeCode()
                ];
            }

            $options[] = [
                'label' => $attributeGroupName,
                'value' => $groupedOptions
            ];
        }

        return $options;
    }

    /**
     * @return \Magento\Eav\Model\ResourceModel\Entity\Attribute\Collection
     */
    public function getAttributesToCheck()
    {
        $excludedGroups = [
            'Prices',
            'Recurring Profile'
        ];

        $excludedAttributes = [
            'sku',
            'status',
            'visibility',
            'old_id',
            'has_options',
            'required_options',
            'options_container',
            'created_at',
            'updated_at',
            'category_ids'
        ];

        $excludedFrontendInputs = [
            'price',
            'image',
            'media_image',
            'gallery'
        ];

        $productEntityType =  $this->typeFactory->create()->loadByCode(Product::ENTITY);
        $attributes = $this->attributeCollectionFactory->create();

        $attributes->setEntityTypeFilter($productEntityType);
        $attributes->join(
            ['eea' => 'eav_entity_attribute'], 'main_table.attribute_id = eea.attribute_id',
            ['group_id' => 'eea.attribute_group_id']
        );
        $attributes->join(
            ['eeg' => 'eav_attribute_group'], 'eea.attribute_group_id = eeg.attribute_group_id',
            ['attribute_group_name' => 'eeg.attribute_group_name']
        );

        $attributes->addFieldToFilter('attribute_group_name', ['nin' => $excludedGroups]);
        $attributes->addFieldToFilter('frontend_input', ['nin' => $excludedFrontendInputs]);
        $attributes->addFieldToFilter('attribute_code', ['nin' => $excludedAttributes]);
        $attributes->getSelect()->group('main_table.attribute_id');
        $attributes->getSelect()->order('attribute_group_name ASC');
        return $attributes;
    }
}